@@ -18,4 +18,8 @@ add column {{ _q | sqlsafe }}{{ column.name | sqlsafe }}{{ _q | sqlsafe }}
         {% else %}
             {% block text_type scoped %} text{% endblock %}
         {% endif %}
+    {% endif %}
+        {% if column.default is not none %}
+        default
+        {% if column.default is string %}'{% endif %}{{ column.default | sqlsafe }}{% if column.default is string %}'{% endif %}
     {% endif %}
\ No newline at end of file
